/*
 * Copyright (c) 2013-2016, Intel Corporation
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of Intel Corporation nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#if !defined(PTI_IMM_DEFS_H)
#define PTI_IMM_DEFS_H

#define PTI_IMM_NONE                                  0
#define PTI_0_IMM_WIDTH_CONST_l2                      1
#define PTI_UIMM8_IMM_WIDTH_CONST_l2                  2
#define PTI_SIMM8_IMM_WIDTH_CONST_l2                  3
#define PTI_SIMMz_IMM_WIDTH_OSZ_NONTERM_EOSZ_l2       4
#define PTI_UIMMv_IMM_WIDTH_OSZ_NONTERM_EOSZ_l2       5
#define PTI_UIMM16_IMM_WIDTH_CONST_l2                 6
#define PTI_RESOLVE_BYREG_IMM_WIDTH_map0x0_op0xf7_l1  7
#define PTI_RESOLVE_BYREG_IMM_WIDTH_map0x0_op0xc7_l1  8
#define PTI_SIMMz_IMM_WIDTH_OSZ_NONTERM_DF64_EOSZ_l2  9
#define PTI_RESOLVE_BYREG_IMM_WIDTH_map0x0_op0xf6_l1 10
#define PTI_IMM_hasimm_map0x0_op0xc8_l1              11
#define PTI_IMM_hasimm_map0x0F_op0x78_l1             12

#endif
